/* ************************************************************************* */
/*                                                                           */
/* Source: Ariplet.java                         |\      _,,,----,            */
/*                                              /,`.-'`'    -.  ;-;;,_       */
/* Author: Oliver Schmid                       |,4-  ) )-,_. ,\ (  `'-'      */
/* Date:   May 2002                           '---''(_/--'  `-'\_)           */
/*                                                                           */
/* Description: With Arithmetic Coding text strings are converted to doubles */
/*       according to their letters' frequencies. The interval [0; 1) is     */
/*       divided into parts proportional to these frequencies.               */
/*          When encoding, the partial interval belonging to the current     */
/*       letter is chosen and becomes the new interval which is divided in   */
/*       the same way as above. When reaching the end or after a certain     */
/*       number of letters the result is written out, being the double       */
/*       writable with the least number of bits within the last intervals    */
/*       bounds.                                                             */
/*          When decoding, the partial interval within whose bounds the re-  */
/*       sult is located is chosen and the letter belonging to the partial   */
/*       interval is written out. The chosen interval becomes the new inter- */
/*       val and is divided as written above. When having decoded the preset */
/*       number of letters the algorithm terminates.                         */
/*                                                                           */
/* ************************************************************************* */



import java.awt.*;
import javax.swing.*;
import java.text.*;



class Ariplet extends Lossless implements Runnable {

    static DecimalFormat f = new DecimalFormat("#.################");

    private String input;

    // Constructor
    public Ariplet(String input) {
	this.input = input;
    }



    public void run() {



	/* ******************   Applet graphics   ****************** */



        // Create JFrame
//      JFrame myFrame = new JFrame("Arithmetic Coding");
//      myFrame.getContentPane().add(this);
//      myFrame.setSize(700, 500);
//      myFrame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
//      myFrame.setVisible(true);

	Color myBackColor = new Color(207, 207, 239);
//      myFrame.setBackground(myBackColor);

	Container myFrame = this;

	// Create Graphics2D with bufferedImage
        Graphics2D graf = this.buffI.createGraphics();
//      graf.setRenderingHint(RenderingHints.KEY_RENDERING,
//                            RenderingHints.VALUE_RENDER_QUALITY);
//      graf.setRenderingHint(RenderingHints.KEY_ANTIALIASING,
//                            RenderingHints.VALUE_ANTIALIAS_ON);
//      graf.setRenderingHint(RenderingHints.KEY_TEXT_ANTIALIASING,
//                            RenderingHints.VALUE_TEXT_ANTIALIAS_ON);
	graf.setColor(myBackColor);
	graf.fillRect(0, 0, 700, 500);
        Font f12 = new Font("Serif", Font.PLAIN, 12);
        FontMetrics fMet12; fMet12 = getFontMetrics(f12);
        Font f18 = new Font("Serif", Font.PLAIN, 18);
        FontMetrics fMet18; fMet18 = getFontMetrics(f18);
        graf.setColor(Color.black); graf.setFont(f12);

        // Set times to wait (in ms)
        int longW = 1000, medW = 250, shrtW = 100;



	/* ******************   Arithmetic intervals   ****************** */



	// Create alphabet
	String alphabet = "";
	for (int i = 0; i < input.length(); i++) {
	    if (alphabet.indexOf(input.charAt(i)) < 0) {
		alphabet += String.valueOf(input.charAt(i));
	    }
	}

	// Create table to count the letters' frequencies
	int[] frequency; frequency = new int[alphabet.length()];
	for (int i = 0; i < frequency.length; i++) {
	    frequency[i] = 0;
	}

	// Count the letters' frequencies
	for (int i = 0; i < input.length(); i++) {
     	    frequency[alphabet.indexOf(input.charAt(i))]++;
	}

	// Cumulate frequency to obtain intervals
	int[] cumulFrequency; cumulFrequency = new int[frequency.length + 1];
	cumulFrequency[0] = 0;
	for (int i = 0; i < frequency.length; i++) {
	    cumulFrequency[i + 1] = cumulFrequency[i] + frequency[i];
	}



	/* *****************   En-/Decoding-coordinates   ***************** */



	// Set coordinates to paint
	int xCStart = 50, xWidth = 300, yStart = 50;
	double ratio = (double)400 / (input.length() + 1);
	int yDistance = (int)ratio; if (ratio > 50) { yDistance = 50; }

	// Create coordinates
	int[] intervalCO; intervalCO = new int[2];
	int[] partInterCO; partInterCO = new int[cumulFrequency.length];
	double[][] limits; limits = new double[input.length() + 1][2];

	for (int i = 0; i < partInterCO.length; i++) {
	    ratio = (double)cumulFrequency[i] /
		cumulFrequency[cumulFrequency.length - 1];
	    ratio *= xWidth;
	    partInterCO[i] = xCStart + (int)ratio;
	}

	intervalCO[0] = xCStart; intervalCO[1] = xCStart + xWidth;

        // Call method for coding text
//  	double codedInput = this.code(input, alphabet, cumulFrequency);
//  	out.println("\nAusgabe: " + f.format(codedInput));



	/* ******************  En-/Decoding-steps   ****************** */



	String bitString = "0.";
  	double lower = 0, upper = 1; // lower and upper limit
  	double returnDouble = 0.5, interval; // interval size

      	for (int i = 0; i < input.length(); i++) {
	    limits[i][0] = lower;
	    limits[i][1] = upper;
	    int hit = alphabet.indexOf(input.charAt(i));
	    interval = upper - lower;
	    upper = lower + interval * cumulFrequency[hit + 1] /
		cumulFrequency[cumulFrequency.length - 1];
	    lower = lower + interval * cumulFrequency[hit] /
		cumulFrequency[cumulFrequency.length - 1];
	}
	limits[input.length()][0] = lower;
	limits[input.length()][1] = upper;

	// Find value to be returned with least number of bits
	if (lower == 0.0) { returnDouble = 0.0;
	} else if (upper == 1.0) { returnDouble = 1.0;
	} // otherwise returnDouble = 0.5 (see above)
	interval = 0.5;
	while ((returnDouble < lower) || (returnDouble > upper)) {
	    interval /= 2;
	    if (returnDouble < lower) {
		returnDouble += interval;
		bitString += "1";
	    } else { // if returnDouble > upper
		returnDouble -= interval;
		bitString += "0";
	    }
	}
	bitString += "1";



	// Call method for decoding coded text
//  	String decodedInput = this.decode(codedInput, alphabet, cumulFrequency,
//  				       input.length());
//  	out.println("\nAusgabe: " + decodedInput);



	/* ******************   Encoding-output   ****************** */



	// Initialize output - encoding
	// Write text
	graf.setColor(Color.blue); graf.setFont(f12);
	graf.drawString(input, 200 - fMet12.stringWidth(input) / 2, 20);
	graf.drawString("Encoding interval", xCStart + xWidth + 20, 20);
	myFrame.repaint();
	try { Thread.sleep(longW); } catch (InterruptedException ex) { }
	int index = alphabet.indexOf(input.charAt(0));
	// Paint first interval
	graf.setColor(Color.black);
	graf.drawLine(intervalCO[0], yStart, intervalCO[1], yStart);
	graf.drawLine(intervalCO[0], yStart,
		      intervalCO[0], yStart + (yDistance / 5));
	graf.drawLine(intervalCO[1], yStart,
		      intervalCO[1], yStart + (yDistance / 5));
	myFrame.repaint();
	try { Thread.sleep(longW); } catch (InterruptedException ex) { }
	// Write first interval
	graf.setColor(Color.blue);
	graf.drawString("[" + String.valueOf(f.format(limits[0][0])) + "; " +
			String.valueOf(f.format(limits[0][1])) + ")",
			xCStart + xWidth + 20,
			yStart + (int)((0.2) * yDistance));
	myFrame.repaint();
	try { Thread.sleep(longW); } catch (InterruptedException ex) { }

	// Loop output - encoding
	for (int i = 0; i < input.length(); i++) {
	    index = alphabet.indexOf(input.charAt(i));

	    // Paint intervals
	    graf.setColor(Color.cyan); graf.setFont(f12);
	    for (int j = 0; j < index; j++) {
		graf.drawString(String.valueOf(alphabet.charAt(j)), (int)
				(partInterCO[j] + partInterCO[j + 1]) / 2 -
				fMet12.stringWidth(String.valueOf(alphabet.charAt(j))) / 2,
				yStart + i * yDistance - 2);
	    }
	    graf.setColor(Color.red); graf.setFont(f18);
	    graf.drawString(String.valueOf(input.charAt(i)), (int)
			    (partInterCO[index] + partInterCO[index + 1]) / 2 -
			    fMet18.stringWidth(String.valueOf(input.charAt(i))) / 2,
			    yStart + i * yDistance - 4);
	    graf.setColor(Color.cyan); graf.setFont(f12);
	    for (int j = index + 1; j < alphabet.length(); j++) {
		graf.drawString(String.valueOf(alphabet.charAt(j)), (int)
				(partInterCO[j] + partInterCO[j + 1]) / 2 -
				fMet12.stringWidth(String.valueOf(alphabet.charAt(j))) / 2,
				yStart + i * yDistance - 2);
	    }
	    graf.setColor(Color.black);
	    for (int j = 0; j < partInterCO.length; j++) {
		graf.drawLine(partInterCO[j],
			      yStart + (20 * i - 1) * yDistance / 20,
			      partInterCO[j],
			      yStart + (20 * i + 1) * yDistance / 20);
	    }
	    myFrame.repaint();
	    try { Thread.sleep(medW); } catch (InterruptedException ex) { }
	    graf.setColor(Color.black);
	    graf.drawLine(partInterCO[index],
			  yStart + (10 * i - 1) * yDistance / 10,
			  partInterCO[index],
			  yStart + (10 * i + 1) * yDistance / 10);
	    graf.drawLine(partInterCO[index + 1],
			  yStart + (10 * i - 1) * yDistance / 10,
			  partInterCO[index + 1],
			  yStart + (10 * i + 1) * yDistance / 10);
	    myFrame.repaint();
	    try { Thread.sleep(medW); } catch (InterruptedException ex) { }

	    // Paint interval projection
	    for (int j = 1; j < 11; j++) {
		graf.setColor(Color.blue);
		graf.drawLine(partInterCO[index], yStart +
			      i * yDistance, partInterCO[index] + (j / 10)
			      * (intervalCO[0] - partInterCO[index]),
			      yStart + (i + j / 10) * yDistance);
		graf.drawLine(partInterCO[index + 1], yStart +
			      i * yDistance, partInterCO[index + 1] + (j / 10)
			      * (intervalCO[1] - partInterCO[index + 1]),
			      yStart + (i + j / 10) * yDistance);

		graf.setColor(Color.black);
		graf.drawLine(intervalCO[0], yStart + i * yDistance,
			      intervalCO[1], yStart + i * yDistance);
		graf.drawLine(intervalCO[0], yStart + i * yDistance,
			      intervalCO[0],
			      yStart + (5 * i + 1) * yDistance / 5);
		graf.drawLine(intervalCO[1], yStart + i * yDistance,
			      intervalCO[1],
			      yStart + (5 * i + 1) * yDistance / 5);
		try { Thread.sleep(shrtW); } catch (InterruptedException ex) {}
		myFrame.repaint();
	    }

	    // Paint new interval
	    graf.setColor(Color.lightGray);
	    graf.drawLine(intervalCO[0], yStart + (i + 1) * yDistance,
			  intervalCO[1], yStart + (i + 1) * yDistance);
	    graf.drawLine(intervalCO[0], yStart + (i + 1) * yDistance,
			  intervalCO[0],
			  yStart + (int)((i + 1.2) * yDistance));
	    graf.drawLine(intervalCO[1], yStart + (i + 1) * yDistance,
			  intervalCO[1],
			  yStart + (int)((i + 1.2) * yDistance));
	    myFrame.repaint();
	    try { Thread.sleep(longW); } catch (InterruptedException ex) { }
	    graf.setColor(Color.blue);
	    graf.drawString("[" + String.valueOf(f.format(limits[i + 1][0])) +
			    "; " + String.valueOf(f.format(limits[i + 1][1])) +
			    ")", xCStart + xWidth + 20,
			    yStart + (int)((i + 1.2) * yDistance));
	    myFrame.repaint();
	    try { Thread.sleep(longW); } catch (InterruptedException ex) { }
	    graf.setColor(Color.black);
	    graf.drawLine(intervalCO[0], yStart + (i + 1) * yDistance,
			  intervalCO[1], yStart + (i + 1) * yDistance);
	    graf.drawLine(intervalCO[0], yStart + (i + 1) * yDistance,
			  intervalCO[0],
			  yStart + (int)((i + 1.2) * yDistance));
	    graf.drawLine(intervalCO[1], yStart + (i + 1)  * yDistance,
			  intervalCO[1],
			  yStart + (int)((i + 1.2) * yDistance));
	    myFrame.repaint();
	    try { Thread.sleep(longW); } catch (InterruptedException ex) { }
	}

	// Write result
	graf.setColor(Color.red); graf.setFont(f12);

	while (fMet12.stringWidth(bitString) >= (xWidth + 20)) {
	    int required = f12.getSize() - 1;
	    f12 = new Font("Serif", Font.PLAIN, required);
	    fMet12 = getFontMetrics(f12);
	    graf.setFont(f12);
	} // to prevent the text from crossing the border...
	graf.drawString(bitString, 200 - fMet12.stringWidth(bitString) / 2,
			yStart + (input.length() + 1) * yDistance);

        f12 = new Font("Serif", Font.PLAIN, 12); fMet12 = getFontMetrics(f12);
	graf.setFont(f12);
	graf.drawString(String.valueOf(returnDouble), xCStart + xWidth + 20,
			yStart + (input.length() + 1) * yDistance);
	myFrame.repaint();
	try { Thread.sleep(10 * longW); } catch (InterruptedException ex) { }



	/* ******************   Intermediate reset   ****************** */



	// Clear Screen
	graf.setColor(myBackColor);
	graf.fillRect(0, 0, 700, 500);
	myFrame.repaint();
	try { Thread.sleep(2 * longW); } catch (InterruptedException ex) { }



	/* ******************   Decoding-output   ****************** */



	// Initialize output - decoding
	// Write text
	graf.setColor(Color.blue); graf.setFont(f12);

	// The X marks the spot
	double spot = (double)
	    fMet12.stringWidth(String.valueOf(returnDouble)) / 2;
	graf.setColor(Color.blue);
	graf.drawLine(200 - (int)spot - 6, 19,
		      200 - (int)spot - 1, 14);
	graf.drawLine(200 - (int)spot - 6, 20,
		      200 - (int)spot, 14);
	graf.drawLine(200 - (int)spot - 5, 20,
		      200 - (int)spot, 15);
	graf.drawLine(200 - (int)spot - 5, 14,
		      200 - (int)spot, 19);
	graf.drawLine(200 - (int)spot - 6, 14,
		      200 - (int)spot, 20);
	graf.drawLine(200 - (int)spot - 6, 15,
		      200 - (int)spot - 1, 20);

	graf.drawString("= " + String.valueOf(returnDouble),
			(int)(200 - spot) + 10, 20);
	graf.drawString("Decoding interval", xCStart + xWidth + 20, 20);
	myFrame.repaint();
	try { Thread.sleep(longW); } catch (InterruptedException ex) { }
	index = alphabet.indexOf(input.charAt(0));
	// Paint first interval
	graf.setColor(Color.black);
	graf.drawLine(intervalCO[0], yStart, intervalCO[1], yStart);
	graf.drawLine(intervalCO[0], yStart,
		      intervalCO[0], yStart + (yDistance / 5));
	graf.drawLine(intervalCO[1], yStart,
		      intervalCO[1], yStart + (yDistance / 5));
	myFrame.repaint();
	try { Thread.sleep(longW); } catch (InterruptedException ex) { }
	// Write first interval
	graf.setColor(Color.blue);
	graf.drawString("[" + String.valueOf(f.format(limits[0][0])) + "; " +
			String.valueOf(f.format(limits[0][1])) + ")",
			xCStart + xWidth + 20,
			yStart + (int)((0.2) * yDistance));
	myFrame.repaint();
	try { Thread.sleep(longW); } catch (InterruptedException ex) { }

	// Loop output - decoding
	for (int i = 0; i < input.length(); i++) {
	    index = alphabet.indexOf(input.charAt(i));

	    graf.setColor(Color.cyan);
	    graf.setFont(f12);
	    for (int j = 0; j < alphabet.length(); j++) {
		graf.drawString(String.valueOf(alphabet.charAt(j)), (int)
				(partInterCO[j] + partInterCO[j + 1]) / 2 -
				fMet12.stringWidth(String.valueOf(alphabet.charAt(j))) / 2,
				yStart + i * yDistance - 2);
	    }
	    myFrame.repaint();
	    try { Thread.sleep(longW); } catch (InterruptedException ex) { }

	    // The X marks the spot
	    spot = xWidth * (returnDouble - limits[i][0]) /
		(limits[i][1] - limits[i][0]);
	    graf.setColor(Color.blue);
	    graf.drawLine(xCStart + (int)spot - 3, yStart + i * yDistance + 2,
			  xCStart + (int)spot + 2, yStart + i * yDistance - 3);
	    graf.drawLine(xCStart + (int)spot - 3, yStart + i * yDistance + 3,
			  xCStart + (int)spot + 3, yStart + i * yDistance - 3);
	    graf.drawLine(xCStart + (int)spot - 2, yStart + i * yDistance + 3,
			  xCStart + (int)spot + 3, yStart + i * yDistance - 2);
	    graf.drawLine(xCStart + (int)spot - 2, yStart + i * yDistance - 3,
			  xCStart + (int)spot + 3, yStart + i * yDistance + 2);
	    graf.drawLine(xCStart + (int)spot - 3, yStart + i * yDistance - 3,
			  xCStart + (int)spot + 3, yStart + i * yDistance + 3);
	    graf.drawLine(xCStart + (int)spot - 3, yStart + i * yDistance - 2,
			  xCStart + (int)spot + 2, yStart + i * yDistance + 3);
	    myFrame.repaint();
	    try { Thread.sleep(longW); } catch (InterruptedException ex) { }

	    // Paint solution
	    graf.setColor(myBackColor); graf.setFont(f12);
	    graf.drawString(String.valueOf(input.charAt(i)), (int)
			    (partInterCO[index] + partInterCO[index + 1]) / 2 -
			    fMet12.stringWidth(String.valueOf(input.charAt(i))) / 2,
			    yStart + i * yDistance - 2);
	    graf.setColor(Color.red); graf.setFont(f18);
	    graf.drawString(String.valueOf(input.charAt(i)), (int)
			    (partInterCO[index] + partInterCO[index + 1]) / 2 -
			    fMet18.stringWidth(String.valueOf(input.charAt(i))) / 2,
			    yStart + i * yDistance - 4);

	    // Paint intervals
	    graf.setColor(Color.black); graf.setFont(f12);
	    for (int j = 0; j < partInterCO.length; j++) {
		graf.drawLine(partInterCO[j],
			      yStart + (20 * i - 1) * yDistance / 20,
			      partInterCO[j],
			      yStart + (20 * i + 1) * yDistance / 20);
	    }
	    myFrame.repaint();
	    try { Thread.sleep(medW); } catch (InterruptedException ex) { }
	    graf.setColor(Color.black);
	    graf.drawLine(partInterCO[index],
			  yStart + (10 * i - 1) * yDistance / 10,
			  partInterCO[index],
			  yStart + (10 * i + 1) * yDistance / 10);
	    graf.drawLine(partInterCO[index + 1],
			  yStart + (10 * i - 1) * yDistance / 10,
			  partInterCO[index + 1],
			  yStart + (10 * i + 1) * yDistance / 10);
	    myFrame.repaint();
	    try { Thread.sleep(medW); } catch (InterruptedException ex) { }

	    // Paint interval projection
	    for (int j = 1; j < 11; j++) {
		graf.setColor(Color.blue);
		graf.drawLine(partInterCO[index], yStart +
			      i * yDistance, partInterCO[index] + (j / 10)
			      * (intervalCO[0] - partInterCO[index]),
			      yStart + (i + j / 10) * yDistance);
		graf.drawLine(partInterCO[index + 1], yStart +
			      i * yDistance, partInterCO[index + 1] + (j / 10)
			      * (intervalCO[1] - partInterCO[index + 1]),
			      yStart + (i + j / 10) * yDistance);

		graf.setColor(Color.black);
		graf.drawLine(intervalCO[0], yStart + i * yDistance,
			      intervalCO[1], yStart + i * yDistance);
		graf.drawLine(intervalCO[0], yStart + i * yDistance,
			      intervalCO[0],
			      yStart + (5 * i + 1) * yDistance / 5);
		graf.drawLine(intervalCO[1], yStart + i * yDistance,
			      intervalCO[1],
			      yStart + (5 * i + 1) * yDistance / 5);
		try { Thread.sleep(shrtW); } catch (InterruptedException ex) {}
		myFrame.repaint();
	    }

	    // Paint new interval
	    graf.setColor(Color.lightGray);
	    graf.drawLine(intervalCO[0], yStart + (i + 1) * yDistance,
			  intervalCO[1], yStart + (i + 1) * yDistance);
	    graf.drawLine(intervalCO[0], yStart + (i + 1) * yDistance,
			  intervalCO[0],
			  yStart + (int)((i + 1.2) * yDistance));
	    graf.drawLine(intervalCO[1], yStart + (i + 1) * yDistance,
			  intervalCO[1],
			  yStart + (int)((i + 1.2) * yDistance));
	    myFrame.repaint();
	    try { Thread.sleep(longW); } catch (InterruptedException ex) { }
	    graf.setColor(Color.blue);
	    graf.drawString("[" + String.valueOf(f.format(limits[i + 1][0])) +
			    "; " + String.valueOf(f.format(limits[i + 1][1])) +
			    ")", xCStart + xWidth + 20,
			    yStart + (int)((i + 1.2) * yDistance));
	    myFrame.repaint();
	    try { Thread.sleep(longW); } catch (InterruptedException ex) { }
	    graf.setColor(Color.black);
	    graf.drawLine(intervalCO[0], yStart + (i + 1) * yDistance,
			  intervalCO[1], yStart + (i + 1) * yDistance);
	    graf.drawLine(intervalCO[0], yStart + (i + 1) * yDistance,
			  intervalCO[0],
			  yStart + (int)((i + 1.2) * yDistance));
	    graf.drawLine(intervalCO[1], yStart + (i + 1)  * yDistance,
			  intervalCO[1],
			  yStart + (int)((i + 1.2) * yDistance));
	    myFrame.repaint();
	    try { Thread.sleep(longW); } catch (InterruptedException ex) { }
	}

	// Write result
	graf.setColor(Color.red); graf.setFont(f12);
	graf.drawString(input, 200 - fMet12.stringWidth(input) / 2,
			yStart + (input.length() + 1) * yDistance);
	myFrame.repaint();
        try { Thread.sleep(60 * longW); } catch (InterruptedException ex) { }



	/* ******************   Final reset   ****************** */



        // Clear Screen
        graf.setColor(myBackColor);
        graf.fillRect(0, 0, 700, 500);
        myFrame.repaint();



    }



    /* ******************   Former methods for textmode   ****************** */



//  public double code(String codableText, String alphabet,
//  		       int[] cumulFrequency) {
//  	double lower = 0, upper = 1; // lower and upper limit
//  	double interval; // interval size
//  	double returnDouble = 0.5; int hit;

//        	for (int i = 0; i < codableText.length(); i++) {
//  	    hit = alphabet.indexOf(codableText.charAt(i));
//  	    out.println("\nZeichen \"" + codableText.charAt(i) + "\" liegt " +
//  			"im relativen Kodierintervall [" +
//  			f.format((double)cumulFrequency[hit] /
//  				 cumulFrequency[cumulFrequency.
//  					       length - 1]) + ", " +
//  			f.format((double)cumulFrequency[hit + 1] /
//  				 cumulFrequency[cumulFrequency.
//  					       length - 1]) + ").");
//  	    interval = upper - lower;
//  	    upper = lower + interval * cumulFrequency[hit + 1] /
//  		cumulFrequency[cumulFrequency.length - 1];
//  	    lower = lower + interval * cumulFrequency[hit] /
//  		cumulFrequency[cumulFrequency.length - 1];
//  	    out.println("Neues Kodierintervall [" + f.format(lower) + ", " +
//  			f.format(upper) + ").");
//  	}

//  	// Find value to be returned with least number of bits
//  	if (lower == 0.0) {
//  	    returnDouble = 0.0;
//  	} else if (upper == 1.0) {
//  	    returnDouble = 1.0;
//  	} // otherwise returnDouble = 0.5 (see above)
//  	interval = 0.5;
//  	while ((returnDouble < lower) || (returnDouble > upper)) {
//  	    interval /= 2;
//  	    if (returnDouble < lower) {
//  		returnDouble += interval;
//  	    } else { // if returnDouble > upper
//  		returnDouble -= interval;
//  	    }
//  	}
//  	return returnDouble;
//  }

//  public String decode(double textableCode, String alphabet,
//  			 int[] cumulFrequency, int codeSize) {
//  	double lower = 0.0, upper = 1.0; // lower and upper limit
//  	double interval = upper - lower; // interval size
//  	String returnText = "";

//  	for (int i = 0; i < codeSize; i++) {
//  	    for (int j = 1; j <= alphabet.length(); j++) {
//  		if (textableCode < (lower + interval * cumulFrequency[j] /
//  				    cumulFrequency[cumulFrequency.length-1])) {
//  		    out.println("\nWert " + f.format(textableCode) +
//  				" liegt im relativen Intervall [" +
//  				f.format((double)cumulFrequency[j - 1] /
//  					 cumulFrequency[cumulFrequency.
//  						       length - 1]) + ", " +
//  				f.format((double)cumulFrequency[j] /
//  					 cumulFrequency[cumulFrequency.
//  						       length - 1]) + ") " +
//  				"und entspricht dem Zeichen \"" +
//  				alphabet.charAt(j - 1) + "\".");
//  		    returnText += String.valueOf(alphabet.charAt(j - 1));
//  		    upper = lower + interval * cumulFrequency[j] /
//  			cumulFrequency[cumulFrequency.length - 1];
//  		    lower = lower + interval * cumulFrequency[j - 1] /
//  			cumulFrequency[cumulFrequency.length - 1];
//  		    interval = upper - lower;
//  		    break;
//  		}
//  	    }
//  	    out.println("Neues Kodierintervall [" + f.format(lower) + ", " +
//  			f.format(upper) + ").");
//  	}
//  	return returnText;
//  }

//  public static void main(String[] args) {
//      Ariplet a = new Ariplet();

//      if (args.length == 0) {
//          a.simulate("ROKOKOKOKOTTEN");
//      } else {
//          a.simulate(args[0].toUpperCase());
//      }
//  }

}

