/* ************************************************************************* */
/*                                                                           */
/* Source: Lemplet.java                         |\      _,,,----,            */
/*                                              /,`.-'`'    -.  ;-;;,_       */
/* Author: Oliver Schmid                       |,4-  ) )-,_. ,\ (  `'-'      */
/* Date:   May 2002                           '---''(_/--'  `-'\_)           */
/*                                                                           */
/* Description: Lempel-Ziv-Algorithms belong to the group of dictionary-     */
/*         based compression. This dictionary is initialized with the single */
/*         characters appearing in the text for encoding as well as for de-  */
/*         coding. As for decoding only the initial dictionary is needed,    */
/*         there is no redundant information needed.                         */
/*            When encoding, in the beginning the prefix is empty. As second */
/*         step the next character is read from the input. Now the algorithm */
/*         tries to find "prefix + character" in the dictionary. When found, */
/*         "prefix + character" become the new prefix, otherwise the code    */
/*         for the prefix is written out, "prefix + character" receives a    */
/*         new entry in the dictionary and the character becomes the new     */
/*         prefix. As long as the end of the input is not reached, second    */
/*         step is next, otherwise the code for the prefix is written out in */
/*         the end.                                                          */
/*            When decoding, in the beginning the prefix is empty. The first */
/*         code, which always belongs to a single letter, is read in and its */
/*         clear text is written out. As first step of the loop this "old    */
/*         code" is always kept in memory. The next code is read in and      */
/*         searched for in the dictionary. When found, itsclear text is      */
/*         written out, the old code is assigned to the prefix and the first */
/*         character of the code is assigned to character. "prefix +         */
/*         character" now receive a new dictionary entry. If not found, the  */
/*         old code is assigned to the prefix and the first character of the */
/*         code is assigned to character. Now "prefix + character" receive a */
/*         new dictionary entry and are written out.                         */
/*                                                                           */
/* ************************************************************************* */



import java.awt.*;
import javax.swing.*;



class Lemplet extends Lossless implements Runnable {

    private String input;

    // Constructor
    public Lemplet(String input) {
	this.input = input;
    }



    public void run() {



	/* ******************   Applet graphics   ****************** */



        // Create JFrame
//      JFrame myFrame = new JFrame("Lempel-Ziv-Welch Coding");
//      myFrame.getContentPane().add(this);
//      myFrame.setSize(700, 500);
//      myFrame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
//      myFrame.setVisible(true);

        Color myBackColor = new Color(207, 207, 239);
//      myFrame.setBackground(myBackColor);

	Container myFrame = this;

	// Create Graphics2D with bufferedImage
        Graphics2D graf = this.buffI.createGraphics();
//      graf.setRenderingHint(RenderingHints.KEY_RENDERING,
//                            RenderingHints.VALUE_RENDER_QUALITY);
//      graf.setRenderingHint(RenderingHints.KEY_ANTIALIASING,
//                            RenderingHints.VALUE_ANTIALIAS_ON);
//      graf.setRenderingHint(RenderingHints.KEY_TEXT_ANTIALIASING,
//                            RenderingHints.VALUE_TEXT_ANTIALIAS_ON);
	graf.setColor(myBackColor);
	graf.fillRect(0, 0, 700, 500);
        graf.setColor(Color.black);
        graf.setFont(new Font("Serif", Font.PLAIN, 12));

	// Set times to wait (in ms)
	int longW = 1000, medW = 500, shrtW = 100;



	/* ******************   LZW-dictionary   ****************** */



	// Create alphabet needed for dictionary
	String alphabet = "";
	for (int i = 0; i < input.length(); i++) {
	    if (alphabet.indexOf(input.charAt(i)) < 0) {
		alphabet += String.valueOf(input.charAt(i));
	    }
	}

	// Create initial dictionary
	String[][] initialDict; initialDict = new String[alphabet.length()][2];
	for (int i = 0; i < alphabet.length(); i++) {
	    initialDict[i][0] = "#" + String.valueOf(i + 1);
	    initialDict[i][1] = String.valueOf(alphabet.charAt(i));
	}

	// Obviously only knowledge of initial dictionary is needed here...



	// Call method for coding text
//  	String[] codedInput = this.code(input, initialDict);
//  	out.print("Codierung von \"" + input + "\": " + codedInput[0]);
//  	for (int i = 1; i < codedInput.length; i++) {
//  	    out.print(" " + codedInput[i]);
//  	}
//  	out.println("\n");



	/* ******************   LZW-Encoding   ****************** */



	String codableText = input;
	String[] Code; // coded text to return

	String[] prefixes; prefixes = new String[codableText.length() + 1];
	String[] characters; characters = new String[codableText.length() + 1];
	String[] dictionary; dictionary = new String[codableText.length() + 1];
	String[] newEntry; newEntry = new String[codableText.length() + 1];
	String[] output; output = new String[codableText.length() + 1];

	for (int i = 0; i < (codableText.length() + 1); i++) {
	    prefixes[i] = ""; characters[i] = ""; dictionary[i] = "";
	    newEntry[i] = ""; output[i] = "";
	}

	// Create dictionary for algorithm use with greater size
	String[][] algoDict;
	algoDict = new String[initialDict.length + codableText.length()][2];
	int lastEntry;



	/* ******************  Encoding-steps   ****************** */



	// 1st step: Initial dictionary contains single chars of input stream
	for (lastEntry = 0; lastEntry < initialDict.length; lastEntry++) {
     	    algoDict[lastEntry][0] = initialDict[lastEntry][0];
     	    algoDict[lastEntry][1] = initialDict[lastEntry][1];
	}

	// 1st step (cont.): prefix is empty
	String prefix = "";

	String character = "", spaceCode = "";
	int hit, numberOfCodes = 0;

	for (int i = 0; i < codableText.length(); i++) {

	    prefixes[i] = prefix;

	    // 2nd step: character := next character of input stream
	    character = codableText.substring(i, i + 1);
	    characters[i] = character;
	    // character = String.valueOf(codableText.charAt(i));
	    
	    for (hit = 0; hit < lastEntry; hit++) {
		if (algoDict[hit][1].compareTo(prefix + character) == 0) {
		    // oldEntry = hit;
		    break;
		}
	    }

	    // 3rd step: Is "prefix + character" in dictionary?
	    if (hit < lastEntry) {
		dictionary[i] = prefix + character + " = #" +
		    String.valueOf(hit + 1);
		prefix += character;
	    } else {
		// Output: Code for prefix
		for (int j = 0; j < lastEntry; j++) {
		    if (algoDict[j][1].compareTo(prefix) == 0) {
			spaceCode += algoDict[j][0];
			output[i] = algoDict[j][0];
			numberOfCodes++;
			break;
		    }
		}
		// Create new dictionary entry
		algoDict[lastEntry][0] = "#" + String.valueOf(lastEntry + 1);
		algoDict[lastEntry][1] = prefix + character;
		dictionary[i] = prefix + character + " = ??";
		newEntry[i] = "#" + String.valueOf(lastEntry + 1) + " = " +
		    prefix + character;
		lastEntry++;
		prefix = character;
	    }
	    // 4th step: If not end of input stream, goto step 2
	}

	prefixes[codableText.length()] = prefix;
	// 4th step: If prefix not empty -> Output: Code for prefix
	for (int i = 0; i < lastEntry; i++) {
	    if (algoDict[i][1].compareTo(prefix) == 0) {
		spaceCode += algoDict[i][0];
		output[codableText.length()] = algoDict[i][0];
		numberOfCodes++;
		break;
	    }
	}

  	Code = new String[numberOfCodes];

	// Convert spaceCode (String) to Code (String[])
        spaceCode = spaceCode.substring(1, spaceCode.length());
	int counter = 0;
        while (spaceCode.length() > 0) {
            int index = spaceCode.indexOf('#');
            if(index > -1) {
                Code[counter++] = "#" + spaceCode.substring(0, index);
                spaceCode = spaceCode.substring(index + 1, spaceCode.length());
            } else {
                Code[counter] = "#" + spaceCode;
		spaceCode = "";
            }
        }



	/* ******************   Encoding-coordinates   ****************** */



	// Set coordinates to paint
	int colX1 = 50, colX2 = 0, colX3, colX4 = 0, colX5 = 0, colX6, colX7;
        int yDistance = (int)(500.0 / (input.length() + 5));
	if (yDistance > 50) {
	    yDistance = 50;
	}
	colX1 = 50;
	for (int i = 0; i < codableText.length(); i++) {
	    if (prefixes[i].length() > colX2) {
		colX2 = prefixes[i].length();
	    }
	    if (dictionary[i].length() > colX4) {
		colX4 = dictionary[i].length();
	    }
	    if (newEntry[i].length() > colX5) {
		colX5 = newEntry[i].length();
	    }
	    
	}
	if (colX2 < 6) {
	    colX2 = colX1 + 50;
	} else {
	    colX2 = colX1 + 10 * colX2;
	}
	colX3 = colX2 + 90;
	if (colX4 < 6) {
	    colX4 = colX3 + 90;
	} else {
	    colX4 = colX3 + 40 + 10 * colX4;
	}
	if (colX5 < 6) {
	    colX5 = colX4 + 90;
	} else {
	    colX5 = colX4 + 40 + 10 * colX5;
	}
	colX6 = colX5 + 90;



	/* ******************   Encoding-output   ****************** */



	// Initialize output - encoding
	graf.setColor(Color.black);
	graf.drawString("Prefix", colX1, yDistance);
	graf.drawString("Character", colX2, yDistance);
	graf.drawString("Searching", colX3, yDistance);
	graf.drawString("New Entry", colX4, yDistance);
	graf.drawString("Output", colX5, yDistance);
	graf.drawString("   Initial", colX6, yDistance);
	graf.drawString("Dictionary", colX6, (int)(1.5 * yDistance));
//  	graf.setColor(Color.blue);
	for (int i = 0; i < initialDict.length; i++) {
	    graf.drawString(initialDict[i][0] + " = "  + initialDict[i][1],
			    colX6, (i + 3) * yDistance);
	}

	myFrame.repaint();
	try { Thread.sleep(longW); } catch (InterruptedException ex) { }


	graf.drawString(characters[0], colX2 + 20, 3 * yDistance);
	myFrame.repaint();
	try { Thread.sleep(medW); } catch (InterruptedException ex) { }

	graf.setColor(Color.green);
	graf.drawString(prefixes[0], colX1 + 10, 3 * yDistance);
	graf.drawString(characters[0], colX2 + 20, 3 * yDistance);
	graf.drawString(dictionary[0], colX3, 3 * yDistance);
	myFrame.repaint();
	try { Thread.sleep(longW); } catch (InterruptedException ex) { }
	graf.setColor(Color.black);
	graf.drawString(characters[0], colX2 + 20, 3 * yDistance);
	myFrame.repaint();
	try { Thread.sleep(longW); } catch (InterruptedException ex) { }

	// Loop output - encoding
	for (int i = 1; i <= codableText.length() - 1; i++) {
	    graf.setColor(Color.black);
	    graf.drawString(prefixes[i], colX1 + 10, (i + 3) * yDistance);
	    myFrame.repaint();
	    try { Thread.sleep(medW); } catch (InterruptedException ex) { }

	    graf.drawString(characters[i], colX2 + 20, (i + 3) * yDistance);
	    myFrame.repaint();
	    try { Thread.sleep(medW); } catch (InterruptedException ex) { }

	    if (dictionary[i].indexOf("#") < 0) {
		graf.setColor(Color.red);
		graf.drawString(prefixes[i], colX1 + 10, (i + 3) * yDistance);
		graf.drawString(characters[i], colX2 + 20,
				(i + 3) * yDistance);
		graf.drawString(dictionary[i], colX3, (i + 3) * yDistance);
		myFrame.repaint();
		try { Thread.sleep(longW); } catch (InterruptedException ex) {}
	    } else {
		graf.setColor(Color.green);
		graf.drawString(prefixes[i], colX1 + 10, (i + 3) * yDistance);
		graf.drawString(characters[i], colX2 + 20,
				(i + 3) * yDistance);
		graf.drawString(dictionary[i], colX3, (i + 3) * yDistance);
		myFrame.repaint();
		try { Thread.sleep(longW); } catch (InterruptedException ex) {}
	    }
	    graf.setColor(Color.black);
	    graf.drawString(prefixes[i], colX1 + 10, (i + 3) * yDistance);
	    graf.drawString(characters[i], colX2 + 20, (i + 3) * yDistance);
	    myFrame.repaint();
	    try { Thread.sleep(longW); } catch (InterruptedException ex) { }

	    if (output[i].length() > 0) {
		graf.setColor(Color.blue);
		graf.drawString(prefixes[i], colX1 + 10, (i + 3) * yDistance);
		myFrame.repaint();
		try { Thread.sleep(medW); } catch (InterruptedException ex) { }
		graf.drawString(output[i], colX5 + 10, (i + 3) * yDistance);
		myFrame.repaint();
		try { Thread.sleep(medW); } catch (InterruptedException ex) { }
		graf.setColor(Color.black);
		graf.drawString(prefixes[i], colX1 + 10, (i + 3) * yDistance);
		myFrame.repaint();
		try { Thread.sleep(longW); } catch (InterruptedException ex) {}

		graf.setColor(Color.green);
		graf.drawString(prefixes[i], colX1 + 10, (i + 3) * yDistance);
		graf.drawString(characters[i], colX2 + 20,
				(i + 3) * yDistance);
		myFrame.repaint();
		try { Thread.sleep(medW); } catch (InterruptedException ex) { }
		graf.drawString(newEntry[i], colX4, (i + 3) * yDistance);
		myFrame.repaint();
		try { Thread.sleep(longW); } catch (InterruptedException ex) {}
		graf.setColor(Color.black);
		graf.drawString(prefixes[i], colX1 + 10, (i + 3) * yDistance);
		graf.drawString(characters[i], colX2 + 20,
				(i + 3) * yDistance);
		graf.drawString(newEntry[i], colX4, (i + 3) * yDistance);
		graf.setColor(myBackColor);
		graf.drawString(dictionary[i], colX3, (i + 3) * yDistance);
		myFrame.repaint();
		try { Thread.sleep(longW); } catch (InterruptedException ex) {}
		try { Thread.sleep(longW); } catch (InterruptedException ex) {}
	    }
	}

	int last = codableText.length();
	graf.setColor(Color.black);
	graf.drawString(prefixes[last], colX1 + 10, (last + 3) * yDistance);
	myFrame.repaint();
	try { Thread.sleep(medW); } catch (InterruptedException ex) { }

	graf.setColor(Color.blue);
	graf.drawString(prefixes[last], colX1 + 10, (last + 3) * yDistance);
	myFrame.repaint();
	try { Thread.sleep(longW); } catch (InterruptedException ex) { }
	graf.drawString(output[last], colX5 + 10, (last + 3) * yDistance);
	myFrame.repaint();
	try { Thread.sleep(medW); } catch (InterruptedException ex) { }
	graf.setColor(Color.black);
	graf.drawString(prefixes[last], colX1 + 10, (last + 3) * yDistance);
	myFrame.repaint();
	try { Thread.sleep(10 * longW); } catch (InterruptedException ex) { }



	/* ******************   Intermediate reset   ****************** */



        // Clear Screen
        graf.setColor(myBackColor);
	graf.fillRect(0, 0, 700, 500);
        myFrame.repaint();
        try { Thread.sleep(2 * longW); } catch (InterruptedException ex) { }



	// Call method for decoding coded text
//  	String decodedInput = this.decode(codedInput, initialDict);
//  	out.print("Decodierung von \"" + codedInput[0]);
//  	for (int i = 1; i < codedInput.length; i++) {
//  	    out.print(" " + codedInput[i]);
//  	}
//  	out.println("\": " + decodedInput + "\n");



	/* ******************   LZW-Decoding   ****************** */



	String returnText = ""; // decoded code to return

	for (int i = 0; i < (codableText.length() + 1); i++) {
	    prefixes[i] = ""; characters[i] = ""; dictionary[i] = "";
	    newEntry[i] = ""; output[i] = "";
	}

	// Create dictionary for algorithm use with greater size
	String[][] algoReDict;
	algoReDict = new String[initialDict.length + Code.length][2];



	/* ******************  Decoding-steps   ****************** */



	// 1st step: initial dictionary contains single chars of input stream
	for (lastEntry = 0; lastEntry < initialDict.length; lastEntry++) {
     	    algoReDict[lastEntry][0] = initialDict[lastEntry][0];
     	    algoReDict[lastEntry][1] = initialDict[lastEntry][1];
	}

  	prefix = ""; character = "";
	String oldText = ""; String clearText = "";
	hit = 0;

	// 2nd & 3rd step: Output: cleartext := first code of input stream
	for (int i = 0; i < lastEntry; i++) {
	    if (algoReDict[i][0].compareTo(Code[0]) == 0) {
		clearText = algoReDict[i][1];
		returnText += clearText;
		dictionary[0] = " = " + algoReDict[i][1];
		output[0] = algoReDict[i][1];
		break;
	    }
	}

  	for (int i = 1; i < Code.length; i++) {

	    // 4th step: Remember clearText in oldText
	    oldText = clearText;

	    // 5th step: clearText := next code of input stream
	    for (hit = 0; hit < lastEntry; hit++) {
		if (algoReDict[hit][0].compareTo(Code[i]) == 0) {
		    clearText = algoReDict[hit][1];
		    break;
		}
	    }

	    // 6th step: Is clearText in dictionary?
	    if(hit < lastEntry) {
		// Output: clearText
		returnText += algoReDict[hit][1];
		dictionary[i] = " = " + algoReDict[hit][1];
		output[i] = algoReDict[hit][1];
		prefix = oldText;
		prefixes[i] = prefix;
		character = clearText.substring(0, 1);
		characters[i] = character;
		// Create new dictionary entry
		algoReDict[lastEntry][0] = "#" + String.valueOf(lastEntry + 1);
		algoReDict[lastEntry][1] = prefix + character;
		newEntry[i] = "#" + String.valueOf(lastEntry + 1) + " = " +
		    prefix + character;
		lastEntry++;
	    } else {
		dictionary[i] = " = ??";
		prefix = oldText;
		prefixes[i] = prefix;
		character = oldText.substring(0, 1);
		characters[i] = character;
		clearText = prefix + character;
		// Create new dictionary entry
		algoReDict[lastEntry][0] = "#" + String.valueOf(lastEntry + 1);
		algoReDict[lastEntry][1] = prefix + character;
		newEntry[i] = "#" + String.valueOf(lastEntry + 1) + " = " +
		    prefix + character;
		output[i] = algoReDict[lastEntry][1];
		lastEntry++;
		// Output: clearText
		returnText += clearText;
	    }
	    // 7th step: If not end of input stream, goto step 4
	}



	/* ******************   Decoding-coordinates   ****************** */



	// Set coordinates to paint
	yDistance = (int)(500.0 / (Code.length + 5));
	if (yDistance > 50) {
	    yDistance = 50;
	}
	colX1 = 50; colX3 = 0; colX4 = 0; colX5 = 0; colX6 = 0; colX7 = 0;
	for (int i = 0; i < codableText.length(); i++) {
	    if (dictionary[i].length() > colX3) {
		colX3 = dictionary[i].length();
	    }
	    if (prefixes[i].length() > colX4) {
		colX4 = prefixes[i].length();
	    }
	    if (newEntry[i].length() > colX6) {
		colX6 = newEntry[i].length();
	    }
	    if (output[i].length() > colX7) {
		colX7 = output[i].length();
	    }
	    
	}
	colX2 = colX1 + 50;
	if (colX3 < 6) {
	    colX3 = colX2 + 90;
	} else {
	    colX3 = colX2 + 40 + 10 * colX3;
	}
	if (colX4 < 6) {
	    colX4 = colX3 + 50;
	} else {
	    colX4 = colX3 + 10 * colX4;
	}
	colX5 = colX4 + 80;
	if (colX6 < 6) {
	    colX6 = colX5 + 90;
	} else {
	    colX6 = colX5 + 40 + 10 * colX6;
	}
	if (colX7 < 6) {
	    colX7 = colX6 + 100;
	} else {
	    colX7 = colX6 + 50 + 10 * colX7;
	}



	/* ******************   Decoding-output   ****************** */



	// Initialize output - decoding
	graf.setColor(Color.black);
	graf.drawString("Code", colX1, yDistance);
	graf.drawString("Searching", colX2, yDistance);
	graf.drawString("Prefix", colX3, yDistance);
	graf.drawString("Character", colX4, yDistance);
	graf.drawString("New Entry", colX5, yDistance);
	graf.drawString("Output", colX6, yDistance);
	graf.drawString("   Initial", colX7, yDistance);
	graf.drawString("Dictionary", colX7, (int)(1.5 * yDistance));
//  	graf.setColor(Color.blue);
	for (int i = 0; i < initialDict.length; i++) {
	    graf.drawString(initialDict[i][0] + " = "  + initialDict[i][1],
			    colX7, (i + 3) * yDistance);
	}

	myFrame.repaint();
	try { Thread.sleep(longW); } catch (InterruptedException ex) { }


	graf.drawString(Code[0], colX1 + 10, 3 * yDistance);
	myFrame.repaint();
	try { Thread.sleep(medW); } catch (InterruptedException ex) { }

	graf.setColor(Color.green);
	graf.drawString(Code[0], colX1 + 10, 3 * yDistance);
	graf.drawString(dictionary[0], colX2, 3 * yDistance);
	myFrame.repaint();
	try { Thread.sleep(longW); } catch (InterruptedException ex) { }
	graf.setColor(Color.black);
	graf.drawString(Code[0], colX1 + 10, 3 * yDistance);
	myFrame.repaint();
	try { Thread.sleep(longW); } catch (InterruptedException ex) { }

	graf.setColor(myBackColor);
	graf.drawString(dictionary[0], colX2, 3 * yDistance);
	graf.setColor(Color.blue);
	graf.drawString(output[0], colX6 + 10, 3 * yDistance);
	myFrame.repaint();
	try { Thread.sleep(longW); } catch (InterruptedException ex) { }

	// Loop output - decoding
	for(int i = 1; i < Code.length; i++) {
	    graf.setColor(Color.black);
	    graf.drawString(Code[i], colX1 + 10, (i + 3) * yDistance);
	    myFrame.repaint();
	    try { Thread.sleep(medW); } catch (InterruptedException ex) { }

	    if(dictionary[i].indexOf("?") < 0) {
		graf.setColor(Color.green);
		graf.drawString(Code[i], colX1 + 10, (i + 3) * yDistance);
		graf.drawString(dictionary[i], colX2, (i + 3) * yDistance);
		myFrame.repaint();
		try { Thread.sleep(longW); } catch (InterruptedException ex) {}
		graf.setColor(Color.blue);
		graf.drawString(output[i], colX6 + 10, (i + 3) * yDistance);
		myFrame.repaint();
		try { Thread.sleep(medW); } catch (InterruptedException ex) { }
		graf.setColor(Color.black);
		graf.drawString(Code[i], colX1 + 10, (i + 3) * yDistance);
		graf.drawString(dictionary[i], colX2, (i + 3) * yDistance);
		myFrame.repaint();
		try { Thread.sleep(longW); } catch (InterruptedException ex) {}
		if (dictionary[i - 1].indexOf("?") < 0) {
		    graf.setColor(Color.green);
		    graf.drawString(dictionary[i - 1], colX2,
				    (i + 2) * yDistance);
		    graf.setColor(Color.black);
		    graf.drawString(dictionary[i - 1].substring(0, 2),
				    colX2, (i + 2) * yDistance);
		    myFrame.repaint();
		    try { Thread.sleep(medW); }
		    catch (InterruptedException ex) { }
		    graf.setColor(Color.green);
		    graf.drawString(prefixes[i], colX3, (i + 3) * yDistance);
		    myFrame.repaint();
		    try { Thread.sleep(medW); }
		    catch (InterruptedException ex) { }
		    graf.setColor(Color.black);
		    graf.drawString(dictionary[i - 1], colX2,
				    (i + 2) * yDistance);
		    myFrame.repaint();
		    try { Thread.sleep(medW); }
		    catch (InterruptedException ex) { }
		} else {
		    graf.setColor(Color.green);
		    graf.drawString(newEntry[i - 1], colX5,
				    (i + 2) * yDistance);
		    graf.setColor(Color.red);
		    graf.drawString(newEntry[i - 1].substring(0, 5),
				    colX5, (i + 2) * yDistance);
		    myFrame.repaint();
		    try { Thread.sleep(medW); }
		    catch (InterruptedException ex) { }
		    graf.setColor(Color.green);
		    graf.drawString(prefixes[i], colX3, (i + 3) * yDistance);
		    myFrame.repaint();
		    try { Thread.sleep(medW); }
		    catch (InterruptedException ex) { }
		    graf.setColor(Color.red);
		    graf.drawString(newEntry[i - 1], colX5,
				    (i + 2) * yDistance);
		    myFrame.repaint();
		    try { Thread.sleep(medW); }
		    catch (InterruptedException ex) { }
		}  
		graf.setColor(Color.green);
		graf.drawString(characters[i], colX4 + 20,
				(i + 3) * yDistance);
		myFrame.repaint();
		try { Thread.sleep(medW); } catch (InterruptedException ex) {}
		graf.drawString(newEntry[i], colX5, (i + 3) * yDistance);
		myFrame.repaint();
		try { Thread.sleep(medW); } catch (InterruptedException ex) {}
		graf.setColor(Color.black);
		graf.drawString(prefixes[i], colX3, (i + 3) * yDistance);
		graf.drawString(characters[i], colX4 + 20,
				(i + 3) * yDistance);
		graf.drawString(newEntry[i], colX5, (i + 3) * yDistance);
		myFrame.repaint();
		try { Thread.sleep(longW); } catch (InterruptedException ex) {}
	    } else {
		graf.setColor(Color.red);
		graf.drawString(Code[i], colX1 + 10, (i + 3) * yDistance);
		graf.drawString(dictionary[i], colX2, (i + 3) * yDistance);
		myFrame.repaint();
		try { Thread.sleep(longW); } catch (InterruptedException ex) {}

		if (dictionary[i - 1].indexOf("?") < 0) {
		    graf.setColor(Color.red);
		    graf.drawString(dictionary[i - 1], colX2,
				    (i + 2) * yDistance);
		    graf.setColor(Color.black);
		    graf.drawString(dictionary[i - 1].substring(0, 2),
				    colX2, (i + 2) * yDistance);
		    myFrame.repaint();
		    try { Thread.sleep(medW); }
		    catch (InterruptedException ex) { }
		    graf.setColor(Color.red);
		    graf.drawString(prefixes[i], colX3, (i + 3) * yDistance);
		    myFrame.repaint();
		    try { Thread.sleep(medW); }
		    catch (InterruptedException ex) { }
		    graf.setColor(Color.black);
		    graf.drawString(dictionary[i - 1], colX2,
				    (i + 2) * yDistance);
		    myFrame.repaint();
		    try { Thread.sleep(medW); }
		    catch (InterruptedException ex) { }
		} else {
		    graf.setColor(Color.black);
		    graf.drawString(newEntry[i - 1], colX5,
				    (i + 2) * yDistance);
		    graf.setColor(Color.red);
		    graf.drawString(newEntry[i - 1].substring(0, 5),
				    colX5, (i + 2) * yDistance);
		    myFrame.repaint();
		    try { Thread.sleep(medW); }
		    catch (InterruptedException ex) { }
		    graf.setColor(Color.red);
		    graf.drawString(prefixes[i], colX3, (i + 3) * yDistance);
		    myFrame.repaint();
		    try { Thread.sleep(medW); }
		    catch (InterruptedException ex) { }
		    graf.setColor(Color.red);
		    graf.drawString(newEntry[i - 1], colX5,
				    (i + 2) * yDistance);
		    myFrame.repaint();
		    try { Thread.sleep(medW); }
		    catch (InterruptedException ex) { }
		}
		graf.setColor(Color.red);
		graf.drawString(characters[i - 1], colX4 + 20,
				(i + 2) * yDistance);
		myFrame.repaint();
		try { Thread.sleep(medW); } catch (InterruptedException ex) { }
		graf.drawString(characters[i], colX4 + 20,
				(i + 3) * yDistance);
		myFrame.repaint();
		try { Thread.sleep(medW); } catch (InterruptedException ex) { }
		graf.drawString(newEntry[i], colX5, (i + 3) * yDistance);
		myFrame.repaint();
		try { Thread.sleep(medW); } catch (InterruptedException ex) { }
		graf.setColor(Color.blue);
		graf.drawString(output[i], colX6 + 10,(i + 3) * yDistance);
		myFrame.repaint();
		try { Thread.sleep(medW); } catch (InterruptedException ex) { }
	    }
	}

        myFrame.repaint();
	try { Thread.sleep(60 * longW); } catch (InterruptedException ex) { }



	/* ******************   Final reset   ****************** */



        // Clear Screen
        graf.setColor(myBackColor);
	graf.fillRect(0, 0, 700, 500);
        myFrame.repaint();

    }



    /* ******************   Former methods for textmode   ****************** */



//  public String[] code(String codableText, String[][] initialDict) {
//  	String[] returnCode; // coded text to return

//  	// Create dictionary for algorithm use with greater size
//  	String[][] algoDict;
//  	algoDict = new String[initialDict.length + codableText.length()][2];
//  	int lastEntry;

//  	// 1st step: Initial dictionary contains single chars of input stream
//  	for (lastEntry = 0; lastEntry < initialDict.length;
//  	     lastEntry++) {
//       	    algoDict[lastEntry][0] = initialDict[lastEntry][0];
//       	    algoDict[lastEntry][1] = initialDict[lastEntry][1];
//  	}

//  	// 1st step (cont.): prefix is empty
//  	String prefix = "";

//  	String character = ""; String spaceCode = "";
//  	int hit; int numberOfCodes = 0;

//  	for (int i = 0; i < codableText.length(); i++) {

//  	    // 2nd step: character := next character of input stream
//  	    character = codableText.substring(i, i + 1);
//  	    // character = String.valueOf(codableText.charAt(i));

//  	    for (hit = 0; hit < lastEntry; hit++) {
//  		if (algoDict[hit][1].compareTo(prefix + character) == 0) {
//  		    // oldEntry = hit;
//  		    break;
//  		}
//  	    }

//  	    // 3rd step: Is "prefix + character" in dictionary?
//  	    if (hit < lastEntry) {
//  		prefix += character;
//  	    } else {
//  		// Output: Code for prefix
//  		for (int j = 0; j < lastEntry; j++) {
//  		    if (algoDict[j][1].compareTo(prefix) == 0) {
//  			spaceCode += algoDict[j][0];
//  			numberOfCodes++;
//  			break;
//  		    }
//  		}
//  		// Create new dictionary entry
//  		algoDict[lastEntry][0] = "#" +
//  		    String.valueOf(lastEntry + 1);
//  		algoDict[lastEntry][1] = prefix + character;
//  		lastEntry++;
//  		prefix = character;
//  	    }
//  	    // 4th step: If not end of input stream, goto step 2
//  	}

//  	// 4th step: If prefix not empty -> Output: Code for prefix
//  	for (int i = 0; i < lastEntry; i++) {
//  	    if (algoDict[i][1].compareTo(prefix) == 0) {
//  		spaceCode += algoDict[i][0];
//  		numberOfCodes++;
//  		break;
//  	    }
//  	}

//  	String[][] finalDict; finalDict = new String[lastEntry][2];

//  	for (int i = 0; i < lastEntry; i++) {
//  	    finalDict[i][0] = algoDict[i][0];
//  	    finalDict[i][1] = algoDict[i][1];
//  	}
//  	for (int i = 0; i < finalDict.length; i++) {
//  	    out.println(finalDict[i][0] + "\t" + finalDict[i][1]);
//  	}

//    	returnCode = new String[numberOfCodes];

//  	// Convert spaceCode (String) to returnCode (String[])
//          spaceCode = spaceCode.substring(1, spaceCode.length());
//  	int counter = 0;
//          while (spaceCode.length() > 0) {
//              int index = spaceCode.indexOf('#');
//              if(index > -1) {
//                  returnCode[counter++] = "#" + spaceCode.substring(0, index);
//                  spaceCode = spaceCode.substring(index + 1, spaceCode.length());
//              } else {
//                  returnCode[counter] = "#" + spaceCode;
//  		spaceCode = "";
//              }
//          }
//  	return returnCode;
//  }

//  public String decode(String[] Code, String[][] initialDict) {
//  	String returnText = ""; // decoded code to return

//  	// Create dictionary for algorithm use with greater size
//  	String[][] algoDict;
//  	algoDict = new String[initialDict.length + Code.length][2];
//  	int lastEntry;

//  	// 1st step: initial dictionary contains single chars of input stream
//  	for (lastEntry = 0; lastEntry < initialDict.length;
//  	     lastEntry++) {
//       	    algoDict[lastEntry][0] = initialDict[lastEntry][0];
//       	    algoDict[lastEntry][1] = initialDict[lastEntry][1];
//  	}

//    	String prefix = ""; String character = "";
//  	String oldText = ""; String clearText = "";
//  	int hit = 0;

//  	out.println("Code\tAusgabe\tNeuerEintrag");

//  	// 2nd & 3rd step: Output: cleartext := first code of input stream
//  	for (int i = 0; i < lastEntry; i++) {
//  	    if (algoDict[i][0].compareTo(Code[0]) == 0) {
//  		clearText = algoDict[i][1];
//  		returnText += clearText;
//  		out.println(algoDict[i][0] + "\t" + clearText + "\t  ---");
//  		break;
//  	    }
//  	}

//    	for (int i = 1; i < Code.length; i++) {

//  	    // 4th step: Remember clearText in oldText
//  	    oldText = clearText;

//  	    // 5th step: clearText := next code of input stream
//  	    for (hit = 0; hit < lastEntry; hit++) {
//  		if (algoDict[hit][0].compareTo(Code[i]) == 0) {
//  		    clearText = algoDict[hit][1];
//  		    break;
//  		}
//  	    }

//  	    // 6th step: Is clearText in dictionary?
//  	    if(hit < lastEntry) {
//  		// Output: clearText
//  		returnText += algoDict[hit][1];
//  		prefix = oldText;
//  		character = clearText.substring(0, 1);
//  		// Create new dictionary entry
//  		algoDict[lastEntry][0] = "#" +
//  		    String.valueOf(lastEntry + 1);
//  		algoDict[lastEntry][1] = prefix + character;
//  		lastEntry++;
//  		out.println(algoDict[hit][0] + "\t" + clearText + "\t#" +
//  		            lastEntry + " = " + prefix + character);
//  	    } else {
//  		prefix = oldText;
//  		character = oldText.substring(0, 1);
//  		clearText = prefix + character;
//  		// Create new dictionary entry
//  		algoDict[lastEntry][0] = "#" +
//  		    String.valueOf(lastEntry + 1);
//  		algoDict[lastEntry][1] = prefix + character;
//  		lastEntry++;
//  		// Output: clearText
//  		returnText += clearText;
//  		out.println(algoDict[hit][0] + "\t" + clearText + "\t#" +
//  			    lastEntry + " = " + prefix + character);
//  	    }
//  	    // 7th step: If not end of input stream, goto step 4
//  	}

//  	String[][] finalDict; finalDict = new String[lastEntry][2];

//  	for (int i = 0; i < lastEntry; i++) {
//  	    finalDict[i][0] = algoDict[i][0];
//  	    finalDict[i][1] = algoDict[i][1];
//  	}
//  	for (int i = 0; i < finalDict.length; i++) {
//  	    out.println(finalDict[i][0] + "\t" + finalDict[i][1]);
//  	}
//  	return returnText;
//  }

//  public static void main(String[] args) {
//      Lemplet l = new Lemplet();

//      if (args.length == 0) {
//          l.simulate("ROKOKOKOKOTTEN");
//      } else {
//          l.simulate(args[0].toUpperCase());
//      }
//  }

}
