/* ************************************************************************* */
/*                                                                           */
/* Source: Startplet.java                       |\      _,,,----,            */
/*                                              /,`.-'`'    -.  ;-;;,_       */
/* Author: Oliver Schmid                       |,4-  ) )-,_. ,\ (  `'-'      */
/* Date:   May 2002                           '---''(_/--'  `-'\_)           */
/*                                                                           */
/* Description: This Applet simulates three compression algorithms belonging */
/*         to the group of lossless compression. The user can choose between */
/*         the following algorithms:                                         */
/*         - Huffman-Code (HC)                                               */
/*         - Lempel-Ziv-Welch-Code (LZW)  and                                */
/*         - Arithmetic Coding (AC)                                          */
/*         Due to their implementation as Threads more than one algorithm    */
/*         can be watched simultaneously. The applets usage is kept simple:  */
/*         1st: Choose a compression algorithm                               */
/*         2nd: Enter a text to encode and decode according to the           */
/*              following rules:                                             */
/*                 - The string must contain between 3 and 18 letters.       */
/*                 - Only ASCII-letters can be entered (A-Z, a-z),           */
/*                      no umlauts or other characters.                      */
/*                 - The string must contain at least 2 different letters.   */
/*                      (HC and AC only)                                     */
/*                 - The string must contain at most 12 different letters.   */
/*                      (AC only)                                            */
/*         3rd: Press START-Button                                           */
/*                                                                           */
/* ************************************************************************* */



import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.border.*;



public class Startplet extends JApplet {



    /* ******************   When started as JApplet   ****************** */

    public void init() {
	Chooseplet cP = new Chooseplet();
	getContentPane().add(cP);
	setSize(270, 270);
	setVisible(true);
    }



    /* ******************   When started as JFrame   ****************** */

    public static void main(String[] args) {
	JFrame startFrame = new JFrame("Choose Applet");
	startFrame.getContentPane().add(new Chooseplet());
	startFrame.pack();
	startFrame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
	startFrame.setVisible(true);
    }
}



/* ******************   JPanel as content of Startplet   ****************** */

class Chooseplet extends JPanel {



    /* ****************   Check entries for applicability   **************** */

    private String checkEntries(String eText, boolean[] chosen1,
				boolean tried2Start) {

	for (int i = 0; i < eText.length(); i++) {
	    int ascii = eText.charAt(i);
	    if (!(((ascii >= 65) && (ascii <= 90)) ||
		  ((ascii >= 97) && (ascii <= 122)))) {
		return("Please enter letters only.");
	    }
	}
	if (tried2Start && (eText.length() < 3)) {
	    return("Enter at least 3 letters.");
	}
	if (tried2Start && (eText.length() > 18)) {
	    return("Enter at most 18 letters.");
	}
	if ((chosen1[1] == false) && tried2Start) {
	    String letters = "";
	    for (int i = 0; i < eText.length(); i++) {
		if (letters.indexOf(eText.charAt(i)) < 0)
		    letters += eText.charAt(i);
	    }
	    if (letters.length() < 2) {
		return("Enter more different letters.");
	    }
	    if ((letters.length() > 12) && (chosen1[2] == true)) {
		return("Enter fewer different letters.");
	    }
	}
	return("");
    }



    /* *****************   JPanel graphics and geometry   ***************** */

    // Constructor
    Chooseplet() {
	final JRadioButton[] choose = {
	    new JRadioButton("  Huffman-Coding  "),
	    new JRadioButton("  Lempel-Ziv-Welch  "),
	    new JRadioButton("  Arithmetic Coding ")
		};
  	final boolean chosen[] = { false, false, false };

	Color myBackColor = new Color(207, 207, 239);
	//        this.setBackground(Color.red);
	Border bor = BorderFactory.createLineBorder(this.getBackground(), 23);
	this.setBorder(bor);
	setLayout(new GridLayout(9, 1));

	final JTextField input = new JTextField(11);
	final JLabel feedback = new JLabel("");

	add(new JLabel("   Please select algorithm:   "));



	/* ******************   Choose algorithm   ****************** */

	ButtonGroup buttGroup = new ButtonGroup();
  	ItemListener listen = new ItemListener() {
		public void itemStateChanged(ItemEvent iE) {
  		    if (iE.getStateChange() == ItemEvent.SELECTED) {
  			for (int i = 0; i < choose.length; i++) {
 			    if (choose[i].isSelected()) {
  				chosen[i] = true;
  			    } else {
  				chosen[i] = false;
  			    }
  			}
			feedback.setText(checkEntries(input.getText(),
						      chosen, false));
  		    }
		}
  	    };
	for (int i = 0; i < choose.length; i++) {
	    choose[i].addItemListener(listen);
	    add(choose[i]);
	    buttGroup.add(choose[i]);
	}

	add(new JLabel(""));
	add(new JLabel("  Please enter text to encode:  "));
	add(input);
	add(feedback);
	JButton start = new JButton("Start new Applet");
	add(start);



	/* ******************   Starting simulation   ****************** */

	start.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent aE) {
		    feedback.setText(checkEntries(input.getText(),
						  chosen, true));
		    if (feedback.getText() == "") {
			if (choose[0].isSelected()) {
			    JFrame hF = new JFrame("Huffman-Code");
			    hF.setSize(700,500);
  			    Huffplet h = new Huffplet(input.getText());
  			    hF.getContentPane().add(h);
  			    hF.setVisible(true);
  			    new Thread(h).start();
			} else if (choose[1].isSelected()) {
			    JFrame lF = new JFrame("Lempel-Ziv-Welch Coding");
			    lF.setSize(700,500);
  			    Lemplet l = new Lemplet(input.getText());
  			    lF.getContentPane().add(l);
  			    lF.setVisible(true);
  			    new Thread(l).start();
			} else if (choose[2].isSelected()) {
			    JFrame aF = new JFrame("Arithmetic Coding");
			    aF.setSize(700,500);
  			    Ariplet a = new Ariplet(input.getText());
  			    aF.getContentPane().add(a);
  			    aF.setVisible(true);
  			    new Thread(a).start();
			} else {
			    feedback.setText("Please choose algorithm.");
			}
		    }
		}
	    });
	
    }



    /* ***************   When started as independant Frame   *************** */

    public static void main(String[] args) {
	JFrame chooseFrame = new JFrame("Choose Applet");
	chooseFrame.getContentPane().add(new Chooseplet());
	chooseFrame.pack();
	chooseFrame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
	chooseFrame.setVisible(true);
    }
}

